# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

from datetime import date


# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'sip'
copyright = '{0} Phil Thompson <phil@riverbankcomputing.com>'.format(
        date.today().year)
author = 'Phil Thompson'
version = 'v6.10.0'


# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = ['myst_parser', 'sphinx.ext.intersphinx']
intersphinx_mapping = {'python': ('https://docs.python.org/3', None)}

#templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']


# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'
#html_static_path = ['_static']


# MyST configuration
myst_enable_extensions = {'deflist'}


# -- Project-specific extensions -----------------------------------------------

def setup(app):
    """ Define roles specific to SIP. """

    app.add_object_type('argument-annotation', 'aanno',
            indextemplate='single: %s (argument annotation)')

    app.add_object_type('class-annotation', 'canno',
            indextemplate='single: %s (class annotation)')

    app.add_object_type('enum-annotation', 'eanno',
            indextemplate='single: %s (enum annotation)')

    app.add_object_type('exception-annotation', 'xanno',
            indextemplate='single: %s (exception annotation)')

    app.add_object_type('function-annotation', 'fanno',
            indextemplate='single: %s (function annotation)')

    app.add_object_type('mapped-type-annotation', 'manno',
            indextemplate='single: %s (mapped type annotation)')

    app.add_object_type('typedef-annotation', 'tanno',
            indextemplate='single: %s (typedef annotation)')

    app.add_object_type('variable-annotation', 'vanno',
            indextemplate='single: %s (variable annotation)')

    app.add_object_type('sip-type', 'stype',
            indextemplate='single: %s (SIP type)')

    app.add_crossref_type('directive', 'directive',
            indextemplate='single: %s (directive)')



# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>


import glob
import inspect
import os
import shutil
import subprocess
import sys
import unittest


class SIPTestCase(unittest.TestCase):
    """ Encapsulate a test case that tests a set of standalone bindings. """

    # The ABI version to use.  None implies the latest major version.  Note
    # that the enum tests are specific to ABI v13.
    #abi_version = None
    abi_version = '13.8'
    #abi_version = '12.15'

    @classmethod
    def setUpClass(cls):
        """ Build a test extension module. """

        # Get the name of the test directory from the file implementing the
        # test case.
        test_dir = os.path.dirname(inspect.getfile(cls))

        # Look for a .sip file in the test directory.
        sip_files = glob.glob(os.path.join(test_dir, '*.sip'))

        if len(sip_files) != 1:
            raise Exception(
                    "expected to find a single .sip file in '{0}'".format(
                            test_dir))

        module_name = os.path.basename(sip_files[0])[:-4]

        # Create a pyproject.toml file.
        pyproject_toml = os.path.join(test_dir, 'pyproject.toml')

        with open(pyproject_toml, 'w') as f:
            f.write(_PYPROJECT_TOML.format(module_name=module_name))

            if cls.abi_version is not None:
                f.write(_ABI_VERSION.format(abi_version=cls.abi_version))

        # Build the extension module.
        cwd = os.getcwd()
        subprocess.run([sys.executable, '-m', 'sipbuild.tools.build', '--verbose'], cwd=test_dir).check_returncode()
        os.chdir(cwd)

        # Move the extension module to the test directory.
        build_dir = os.path.join(test_dir, 'build')

        # The distutils and setuptools builders leave the module in different
        # places.
        for subdirs in ((module_name, ), (module_name, 'build', 'lib*')):
            module_pattern = [build_dir]
            module_pattern.extend(subdirs)
            module_pattern.append(
                    module_name + '*.pyd' if sys.platform == 'win32' else '*.so')

            module_path = glob.glob(os.path.join(*module_pattern))
            if len(module_path) != 0:
                break
        else:
            raise Exception(
                    "no '{0}' extension module was built".format(module_name))

        if len(module_path) != 1:
            raise Exception(
                    "unable to determine file name of the '{0}' extension module".format(module_name))

        module_path = module_path[0]
        module_impl = os.path.join(test_dir, os.path.basename(module_path))

        # On Windows the module may be lying around from a previous run.
        try:
            os.remove(module_impl)
        except:
            pass

        os.rename(module_path, module_impl)

        # Remove the build artifacts.
        os.remove(pyproject_toml)
        shutil.rmtree(build_dir, ignore_errors=True)

        # Provide tearDownClass() with the values it needs to tidy up.
        cls._module_name = module_name
        cls._module_impl = module_impl

    @classmethod
    def tearDownClass(cls):
        """ Unload the module and remove the extension module. """

        try:
            del sys.modules[cls._module_name]
        except KeyError:
            pass

        # This will fail on Windows (probably because the module is loaded).
        try:
            os.remove(cls._module_impl)
        except:
            pass

    def install_hook(self):
        """ Install a hook that will catch any exceptions raised by a Python
        reimplementation of a C++ virtual.
        """

        # Clear the saved exception.
        self._exc = None

        # Save the old hook and install the new one.
        self._old_hook = sys.excepthook
        sys.excepthook = self._hook

    def uninstall_hook(self):
        """ Restore the original exception hook and re-raise any exception
        raised by a virtual reimplementation.
        """

        sys.excepthook = self._old_hook

        if self._exc is not None:
            raise self._exc

    def _hook(self, xtype, xvalue, xtb):
        """ The replacement exceptionhook. """

        # Save the exception for later.
        self._exc = xvalue


# The prototype pyproject.toml file.
_PYPROJECT_TOML = """
[build-system]
requires = ["sip >=6"]
build-backend = "sipbuild.api"

[project]
name = "{module_name}"

[tool.sip.project]
minimum-macos-version = "10.9"
"""

_ABI_VERSION = """
abi-version = "{abi_version}"
"""



# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>


from utils import SIPTestCase


class IntConvertorsTestCase(SIPTestCase):
    """ Test the integer convertors. """

    @classmethod
    def setUpClass(cls):
        """ Set up the test case. """

        super().setUpClass()

        from .int_convertors import IntConvertors

        # Compute the various test values based on the native sizes.
        cls.CHAR_LOWER = IntConvertors.char_lower()
        cls.CHAR_UPPER = IntConvertors.char_upper()
        cls.SIGNED_CHAR_LOWER, cls.SIGNED_CHAR_UPPER = cls._signed_bounds(
                IntConvertors.signed_char_sizeof())
        cls.SHORT_LOWER, cls.SHORT_UPPER = cls._signed_bounds(
                IntConvertors.short_sizeof())
        cls.INT_LOWER, cls.INT_UPPER = cls._signed_bounds(
                IntConvertors.int_sizeof())
        cls.LONG_LOWER, cls.LONG_UPPER = cls._signed_bounds(
                IntConvertors.long_sizeof())
        cls.LONG_LONG_LOWER, cls.LONG_LONG_UPPER = cls._signed_bounds(
                IntConvertors.long_long_sizeof())
        cls.UNSIGNED_CHAR_UPPER = cls._unsigned_upper_bound(
                IntConvertors.unsigned_char_sizeof())
        cls.UNSIGNED_SHORT_UPPER = cls._unsigned_upper_bound(
                IntConvertors.unsigned_short_sizeof())
        cls.UNSIGNED_INT_UPPER = cls._unsigned_upper_bound(
                IntConvertors.unsigned_int_sizeof())
        cls.UNSIGNED_LONG_UPPER = cls._unsigned_upper_bound(
                IntConvertors.unsigned_long_sizeof())
        cls.UNSIGNED_LONG_LONG_UPPER = cls._unsigned_upper_bound(
                IntConvertors.unsigned_long_long_sizeof())

        class LimitsFixture(IntConvertors):
            """ The base test fixture for those implementing a range of values.
            """

            def __init__(self, limits):
                """ Initialise the object. """

                super().__init__()

                self.limits = limits

        class ValidLowerFixture(LimitsFixture):
            """ A fixture for testing the lower bound of non-overflowing signed
            values.
            """

            def char_virt(self):
                return self.limits.CHAR_LOWER

            def signed_char_virt(self):
                return self.limits.SIGNED_CHAR_LOWER

            def short_virt(self):
                return self.limits.SHORT_LOWER

            def int_virt(self):
                return self.limits.INT_LOWER

            def long_virt(self):
                return self.limits.LONG_LOWER

            def long_long_virt(self):
                return self.limits.LONG_LONG_LOWER

        cls.valid_lower_fixture = ValidLowerFixture(cls)

        class ValidUpperFixture(LimitsFixture):
            """ A fixture for testing the upper bound of non-overflowing
            values.
            """

            def char_virt(self):
                return self.limits.CHAR_UPPER

            def signed_char_virt(self):
                return self.limits.SIGNED_CHAR_UPPER

            def short_virt(self):
                return self.limits.SHORT_UPPER

            def int_virt(self):
                return self.limits.INT_UPPER

            def long_virt(self):
                return self.limits.LONG_UPPER

            def long_long_virt(self):
                return self.limits.LONG_LONG_UPPER

            def unsigned_char_virt(self):
                return self.limits.UNSIGNED_CHAR_UPPER

            def unsigned_short_virt(self):
                return self.limits.UNSIGNED_SHORT_UPPER

            def unsigned_int_virt(self):
                return self.limits.UNSIGNED_INT_UPPER

            def unsigned_long_virt(self):
                return self.limits.UNSIGNED_LONG_UPPER

            def unsigned_long_long_virt(self):
                return self.limits.UNSIGNED_LONG_LONG_UPPER

        cls.valid_upper_fixture = ValidUpperFixture(cls)

        class InvalidFixture(IntConvertors):
            """ A fixture for testing invalid values. """

            def bool_virt(self):
                return '0'

            def char_virt(self):
                return '0'

            def signed_char_virt(self):
                return '0'

            def short_virt(self):
                return '0'

            def int_virt(self):
                return '0'

            def long_virt(self):
                return '0'

            def long_long_virt(self):
                return '0'

            def unsigned_char_virt(self):
                return '0'

            def unsigned_short_virt(self):
                return '0'

            def unsigned_int_virt(self):
                return '0'

            def unsigned_long_virt(self):
                return '0'

            def unsigned_long_long_virt(self):
                return '0'

        cls.invalid_fixture = InvalidFixture()

        class OverflowLowerFixture(LimitsFixture):
            """ A fixture for testing the lower bound of overflowing signed
            values.
            """

            def char_virt(self):
                return self.limits.CHAR_LOWER - 1

            def signed_char_virt(self):
                return self.limits.SIGNED_CHAR_LOWER - 1

            def short_virt(self):
                return self.limits.SHORT_LOWER - 1

            def int_virt(self):
                return self.limits.INT_LOWER - 1

            def long_virt(self):
                return self.limits.LONG_LOWER - 1

            def long_long_virt(self):
                return self.limits.LONG_LONG_LOWER - 1

        cls.overflow_lower_fixture = OverflowLowerFixture(cls)

        class OverflowUpperFixture(LimitsFixture):
            """ A fixture for testing the upper bound of overflowing values. """

            def char_virt(self):
                return self.limits.CHAR_UPPER + 1

            def signed_char_virt(self):
                return self.limits.SIGNED_CHAR_UPPER + 1

            def short_virt(self):
                return self.limits.SHORT_UPPER + 1

            def int_virt(self):
                return self.limits.INT_UPPER + 1

            def long_virt(self):
                return self.limits.LONG_UPPER + 1

            def long_long_virt(self):
                return self.limits.LONG_LONG_UPPER + 1

            def unsigned_char_virt(self):
                return self.limits.UNSIGNED_CHAR_UPPER + 1

            def unsigned_short_virt(self):
                return self.limits.UNSIGNED_SHORT_UPPER + 1

            def unsigned_int_virt(self):
                return self.limits.UNSIGNED_INT_UPPER + 1

            def unsigned_long_virt(self):
                return self.limits.UNSIGNED_LONG_UPPER + 1

            def unsigned_long_long_virt(self):
                return self.limits.UNSIGNED_LONG_LONG_UPPER + 1

        cls.overflow_upper_fixture = OverflowUpperFixture(cls)

        class BoolFixture(IntConvertors):
            """ A fixture for testing valid boolean values. """

            def __init__(self, value):
                """ Initialise the object. """

                super().__init__()

                self._value = value

            def bool_virt(self):
                return self._value

        cls.true_fixture = BoolFixture(True)
        cls.false_fixture = BoolFixture(False)
        cls.nonzero_fixture = BoolFixture(-1)
        cls.zero_fixture = BoolFixture(0)

    @staticmethod
    def _signed_bounds(nrbytes):
        """ Return the range of values for a number of bytes representing a
        signed value.
        """

        v = 1 << ((nrbytes * 8) - 1)

        return -v, v - 1

    @staticmethod
    def _unsigned_upper_bound(nrbytes):
        """ Return the upper bound for a number of bytes representing an
        unsigned value.
        """

        return (1 << (nrbytes * 8)) - 1

    @classmethod
    def tearDownClass(cls):
        """ Tear down the test case. """

        # Remove all references to the extension module so that the superclass
        # can unload it.
        del cls.valid_lower_fixture
        del cls.valid_upper_fixture

        del cls.invalid_fixture

        del cls.overflow_lower_fixture
        del cls.overflow_upper_fixture

        del cls.true_fixture
        del cls.false_fixture
        del cls.nonzero_fixture
        del cls.zero_fixture

        super().tearDownClass()

    ###########################################################################
    # The following test for valid values.
    ###########################################################################

    def test_char_get_lower_valid(self):
        """ char virtual result lower bound. """

        self.assertEqual(self.valid_lower_fixture.char_get(), self.CHAR_LOWER)

    def test_char_get_upper_valid(self):
        """ char virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.char_get(), self.CHAR_UPPER)

    def test_char_set_lower_valid(self):
        """ char function argument lower bound. """

        self.valid_lower_fixture.char_set(self.CHAR_LOWER)

    def test_char_set_upper_valid(self):
        """ char function argument upper bound. """

        self.valid_upper_fixture.char_set(self.CHAR_UPPER)

    def test_char_var_lower_valid(self):
        """ char instance variable lower bound. """

        self.valid_lower_fixture.char_var = self.CHAR_LOWER

    def test_char_var_upper_valid(self):
        """ char instance variable upper bound. """

        self.valid_upper_fixture.char_var = self.CHAR_UPPER

    def test_signed_char_get_lower_valid(self):
        """ signed char virtual result lower bound. """

        self.assertEqual(self.valid_lower_fixture.signed_char_get(),
                self.SIGNED_CHAR_LOWER)

    def test_signed_char_get_upper_valid(self):
        """ signed char virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.signed_char_get(),
                self.SIGNED_CHAR_UPPER)

    def test_signed_char_set_lower_valid(self):
        """ signed char function argument lower bound. """

        self.valid_lower_fixture.signed_char_set(self.SIGNED_CHAR_LOWER)

    def test_signed_char_set_upper_valid(self):
        """ signed char function argument upper bound. """

        self.valid_upper_fixture.signed_char_set(self.SIGNED_CHAR_UPPER)

    def test_signed_char_var_lower_valid(self):
        """ signed char instance variable lower bound. """

        self.valid_lower_fixture.signed_char_var = self.SIGNED_CHAR_LOWER

    def test_signed_char_var_upper_valid(self):
        """ signed char instance variable upper bound. """

        self.valid_upper_fixture.signed_char_var = self.SIGNED_CHAR_UPPER

    def test_short_get_lower_valid(self):
        """ short virtual result lower bound. """

        self.assertEqual(self.valid_lower_fixture.short_get(),
                self.SHORT_LOWER)

    def test_short_get_upper_valid(self):
        """ short virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.short_get(),
                self.SHORT_UPPER)

    def test_short_set_lower_valid(self):
        """ short function argument lower bound. """

        self.valid_lower_fixture.short_set(self.SHORT_LOWER)

    def test_short_set_upper_valid(self):
        """ short function argument upper bound. """

        self.valid_upper_fixture.short_set(self.SHORT_UPPER)

    def test_short_var_lower_valid(self):
        """ short instance variable lower bound. """

        self.valid_lower_fixture.short_var = self.SHORT_LOWER

    def test_short_var_upper_valid(self):
        """ short instance variable upper bound. """

        self.valid_upper_fixture.short_var = self.SHORT_UPPER

    def test_int_get_lower_valid(self):
        """ int virtual result lower bound. """

        self.assertEqual(self.valid_lower_fixture.int_get(), self.INT_LOWER)

    def test_int_get_upper_valid(self):
        """ int virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.int_get(), self.INT_UPPER)

    def test_int_set_lower_valid(self):
        """ int function argument lower bound. """

        self.valid_lower_fixture.int_set(self.INT_LOWER)

    def test_int_set_upper_valid(self):
        """ int function argument upper bound. """

        self.valid_upper_fixture.int_set(self.INT_UPPER)

    def test_int_var_lower_valid(self):
        """ int instance variable lower bound. """

        self.valid_lower_fixture.int_var = self.INT_LOWER

    def test_int_var_upper_valid(self):
        """ int instance variable upper bound. """

        self.valid_upper_fixture.int_var = self.INT_UPPER

    def test_long_get_lower_valid(self):
        """ long virtual result lower bound. """

        self.assertEqual(self.valid_lower_fixture.long_get(), self.LONG_LOWER)

    def test_long_get_upper_valid(self):
        """ long virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.long_get(), self.LONG_UPPER)

    def test_long_set_lower_valid(self):
        """ long function argument lower bound. """

        self.valid_lower_fixture.long_set(self.LONG_LOWER)

    def test_long_set_upper_valid(self):
        """ long function argument upper bound. """

        self.valid_upper_fixture.long_set(self.LONG_UPPER)

    def test_long_var_lower_valid(self):
        """ long instance variable lower bound. """

        self.valid_lower_fixture.long_var = self.LONG_LOWER

    def test_long_var_upper_valid(self):
        """ long instance variable upper bound. """

        self.valid_upper_fixture.long_var = self.LONG_UPPER

    def test_long_long_get_lower_valid(self):
        """ long long virtual result lower bound. """

        self.assertEqual(self.valid_lower_fixture.long_long_get(),
                self.LONG_LONG_LOWER)

    def test_long_long_get_upper_valid(self):
        """ long long virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.long_long_get(),
                self.LONG_LONG_UPPER)

    def test_long_long_set_lower_valid(self):
        """ long long function argument lower bound. """

        self.valid_lower_fixture.long_long_set(self.LONG_LONG_LOWER)

    def test_long_long_set_upper_valid(self):
        """ long long function argument upper bound. """

        self.valid_upper_fixture.long_long_set(self.LONG_LONG_UPPER)

    def test_long_long_var_lower_valid(self):
        """ long long instance variable lower bound. """

        self.valid_lower_fixture.long_long_var = self.LONG_LONG_LOWER

    def test_long_long_var_upper_valid(self):
        """ long long instance variable upper bound. """

        self.valid_upper_fixture.long_long_var = self.LONG_LONG_UPPER

    def test_unsigned_char_get_upper_valid(self):
        """ unsigned char virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.unsigned_char_get(),
                self.UNSIGNED_CHAR_UPPER)

    def test_unsigned_char_set_upper_valid(self):
        """ unsigned char function argument upper bound. """

        self.valid_upper_fixture.unsigned_char_set(self.UNSIGNED_CHAR_UPPER)

    def test_unsigned_char_var_upper_valid(self):
        """ unsigned char instance variable upper bound. """

        self.valid_upper_fixture.unsigned_char_var = self.UNSIGNED_CHAR_UPPER

    def test_unsigned_short_get_upper_valid(self):
        """ unsigned short virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.unsigned_short_get(),
                self.UNSIGNED_SHORT_UPPER)

    def test_unsigned_short_set_upper_valid(self):
        """ unsigned short function argument upper bound. """

        self.valid_upper_fixture.unsigned_short_set(self.UNSIGNED_SHORT_UPPER)

    def test_unsigned_short_var_upper_valid(self):
        """ unsigned short instance variable upper bound. """

        self.valid_upper_fixture.unsigned_short_var = self.UNSIGNED_SHORT_UPPER

    def test_unsigned_int_get_upper_valid(self):
        """ unsigned int virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.unsigned_int_get(),
                self.UNSIGNED_INT_UPPER)

    def test_unsigned_int_set_upper_valid(self):
        """ unsigned int function argument upper bound. """

        self.valid_upper_fixture.unsigned_int_set(self.UNSIGNED_INT_UPPER)

    def test_unsigned_int_var_upper_valid(self):
        """ unsigned int instance variable upper bound. """

        self.valid_upper_fixture.unsigned_int_var = self.UNSIGNED_INT_UPPER

    def test_unsigned_long_get_upper_valid(self):
        """ unsigned long virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.unsigned_long_get(),
                self.UNSIGNED_LONG_UPPER)

    def test_unsigned_long_set_upper_valid(self):
        """ unsigned long function argument upper bound. """

        self.valid_upper_fixture.unsigned_long_set(self.UNSIGNED_LONG_UPPER)

    def test_unsigned_long_var_upper_valid(self):
        """ unsigned long instance variable upper bound. """

        self.valid_upper_fixture.unsigned_long_var = self.UNSIGNED_LONG_UPPER

    def test_unsigned_long_long_get_upper_valid(self):
        """ unsigned long long virtual result upper bound. """

        self.assertEqual(self.valid_upper_fixture.unsigned_long_long_get(),
                self.UNSIGNED_LONG_LONG_UPPER)

    def test_unsigned_long_long_set_upper_valid(self):
        """ unsigned long long function argument upper bound. """

        self.valid_upper_fixture.unsigned_long_long_set(
                self.UNSIGNED_LONG_LONG_UPPER)

    def test_unsigned_long_long_var_upper_valid(self):
        """ unsigned long long instance variable upper bound. """

        self.valid_upper_fixture.unsigned_long_long_var = self.UNSIGNED_LONG_LONG_UPPER

    ###########################################################################
    # The following test for invalid values.
    ###########################################################################

    def test_bool_get_invalid(self):
        """ bool virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.bool_get()
            self.uninstall_hook()

    def test_bool_set_invalid(self):
        """ bool function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.bool_set('0')

    def test_bool_var_invalid(self):
        """ bool instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.bool_var = '0'

    def test_char_get_invalid(self):
        """ char virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.char_get()
            self.uninstall_hook()

    def test_char_set_invalid(self):
        """ char function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.char_set('0')

    def test_char_var_invalid(self):
        """ char instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.char_var = '0'

    def test_signed_char_get_invalid(self):
        """ signed char virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.signed_char_get()
            self.uninstall_hook()

    def test_signed_char_set_invalid(self):
        """ signed char function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.signed_char_set('0')

    def test_signed_char_var_invalid(self):
        """ signed char instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.signed_char_var = '0'

    def test_short_get_invalid(self):
        """ short virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.short_get()
            self.uninstall_hook()

    def test_short_set_invalid(self):
        """ short function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.short_set('0')

    def test_short_var_invalid(self):
        """ short instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.short_var = '0'

    def test_int_get_invalid(self):
        """ int virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.int_get()
            self.uninstall_hook()

    def test_int_set_invalid(self):
        """ int function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.int_set('0')

    def test_int_var_invalid(self):
        """ int instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.int_var = '0'

    def test_long_get_invalid(self):
        """ long virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.long_get()
            self.uninstall_hook()

    def test_long_set_invalid(self):
        """ long function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.long_set('0')

    def test_long_var_invalid(self):
        """ long instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.long_var = '0'

    def test_long_long_get_invalid(self):
        """ long long virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.long_long_get()
            self.uninstall_hook()

    def test_long_long_set_invalid(self):
        """ long long function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.long_long_set('0')

    def test_long_long_var_invalid(self):
        """ long long instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.long_long_var = '0'

    def test_unsigned_char_get_invalid(self):
        """ unsigned char virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.unsigned_char_get()
            self.uninstall_hook()

    def test_unsigned_char_set_invalid(self):
        """ unsigned char function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.unsigned_char_set('0')

    def test_unsigned_char_var_invalid(self):
        """ unsigned char instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.unsigned_char_var = '0'

    def test_unsigned_short_get_invalid(self):
        """ unsigned short virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.unsigned_short_get()
            self.uninstall_hook()

    def test_unsigned_short_set_invalid(self):
        """ unsigned short function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.unsigned_short_set('0')

    def test_unsigned_short_var_invalid(self):
        """ unsigned short instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.unsigned_short_var = '0'

    def test_unsigned_int_get_invalid(self):
        """ unsigned int virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.unsigned_int_get()
            self.uninstall_hook()

    def test_unsigned_int_set_invalid(self):
        """ unsigned int function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.unsigned_int_set('0')

    def test_unsigned_int_var_invalid(self):
        """ unsigned int instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.unsigned_int_var = '0'

    def test_unsigned_long_get_invalid(self):
        """ unsigned long virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.unsigned_long_get()
            self.uninstall_hook()

    def test_unsigned_long_set_invalid(self):
        """ unsigned long function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.unsigned_long_set('0')

    def test_unsigned_long_var_invalid(self):
        """ unsigned long instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.unsigned_long_var = '0'

    def test_unsigned_long_long_get_invalid(self):
        """ unsigned long long virtual result. """

        with self.assertRaises(TypeError):
            self.install_hook()
            self.invalid_fixture.unsigned_long_long_get()
            self.uninstall_hook()

    def test_unsigned_long_long_set_invalid(self):
        """ unsigned long long function argument. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.unsigned_long_long_set('0')

    def test_unsigned_long_long_var_invalid(self):
        """ unsigned long long instance variable. """

        with self.assertRaises(TypeError):
            self.invalid_fixture.unsigned_long_long_var = '0'

    ###########################################################################
    # The following test for under/overlowing values.
    ###########################################################################

    def test_char_get_lower_overflow(self):
        """ char virtual result lower bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_lower_fixture.char_get()
            self.uninstall_hook()

    def test_char_get_upper_overflow(self):
        """ char virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.char_get()
            self.uninstall_hook()

    def test_char_set_lower_overflow(self):
        """ char function argument lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.char_set(self.CHAR_LOWER - 1)

    def test_char_set_upper_overflow(self):
        """ char function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.char_set(self.CHAR_UPPER + 1)

    def test_char_var_lower_overflow(self):
        """ char instance variable lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.char_var = self.CHAR_LOWER - 1

    def test_char_var_upper_overflow(self):
        """ char instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.char_var = self.CHAR_UPPER + 1

    def test_signed_char_get_lower_overflow(self):
        """ signed char virtual result lower bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_lower_fixture.signed_char_get()
            self.uninstall_hook()

    def test_signed_char_get_upper_overflow(self):
        """ signed char virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.signed_char_get()
            self.uninstall_hook()

    def test_signed_char_set_lower_overflow(self):
        """ signed char function argument lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.signed_char_set(
                    self.SIGNED_CHAR_LOWER - 1)

    def test_signed_char_set_upper_overflow(self):
        """ signed char function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.signed_char_set(
                    self.SIGNED_CHAR_UPPER + 1)

    def test_signed_char_var_lower_overflow(self):
        """ signed char instance variable lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.signed_char_var = self.SIGNED_CHAR_LOWER - 1

    def test_signed_char_var_upper_overflow(self):
        """ signed char instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.signed_char_var = self.SIGNED_CHAR_UPPER + 1

    def test_short_get_lower_overflow(self):
        """ short virtual result lower bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_lower_fixture.short_get()
            self.uninstall_hook()

    def test_short_get_upper_overflow(self):
        """ short virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.short_get()
            self.uninstall_hook()

    def test_short_set_lower_overflow(self):
        """ short function argument lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.short_set(self.SHORT_LOWER - 1)

    def test_short_set_upper_overflow(self):
        """ short function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.short_set(self.SHORT_UPPER + 1)

    def test_short_var_lower_overflow(self):
        """ short instance variable lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.short_var = self.SHORT_LOWER - 1

    def test_short_var_upper_overflow(self):
        """ short instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.short_var = self.SHORT_UPPER + 1

    def test_int_get_lower_overflow(self):
        """ int virtual result lower bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_lower_fixture.int_get()
            self.uninstall_hook()

    def test_int_get_upper_overflow(self):
        """ int virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.int_get()
            self.uninstall_hook()

    def test_int_set_lower_overflow(self):
        """ int function argument lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.int_set(self.INT_LOWER - 1)

    def test_int_set_upper_overflow(self):
        """ int function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.int_set(self.INT_UPPER + 1)

    def test_int_var_lower_overflow(self):
        """ int instance variable lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.int_var = self.INT_LOWER - 1

    def test_int_var_upper_overflow(self):
        """ int instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.int_var = self.INT_UPPER + 1

    def test_long_get_lower_overflow(self):
        """ long virtual result lower bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_lower_fixture.long_get()
            self.uninstall_hook()

    def test_long_get_upper_overflow(self):
        """ long virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.long_get()
            self.uninstall_hook()

    def test_long_set_lower_overflow(self):
        """ long function argument lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.long_set(self.LONG_LOWER - 1)

    def test_long_set_upper_overflow(self):
        """ long function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.long_set(self.LONG_UPPER + 1)

    def test_long_var_lower_overflow(self):
        """ long instance variable lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.long_var = self.LONG_LOWER - 1

    def test_long_var_upper_overflow(self):
        """ long instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.long_var = self.LONG_UPPER + 1

    def test_long_long_get_lower_overflow(self):
        """ long long virtual result lower bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_lower_fixture.long_long_get()
            self.uninstall_hook()

    def test_long_long_get_upper_overflow(self):
        """ long long virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.long_long_get()
            self.uninstall_hook()

    def test_long_long_set_lower_overflow(self):
        """ long long function argument lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.long_long_set(self.LONG_LONG_LOWER - 1)

    def test_long_long_set_upper_overflow(self):
        """ long long function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.long_long_set(self.LONG_LONG_UPPER + 1)

    def test_long_long_var_lower_overflow(self):
        """ long long instance variable lower bound. """

        with self.assertRaises(OverflowError):
            self.overflow_lower_fixture.long_long_var = self.LONG_LONG_LOWER - 1

    def test_long_long_var_upper_overflow(self):
        """ long long instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.long_long_var = self.LONG_LONG_UPPER + 1

    def test_unsigned_char_get_upper_overflow(self):
        """ unsigned char virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.unsigned_char_get()
            self.uninstall_hook()

    def test_unsigned_char_set_upper_overflow(self):
        """ unsigned char function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.unsigned_char_set(
                    self.UNSIGNED_CHAR_UPPER + 1)

    def test_unsigned_char_var_upper_overflow(self):
        """ unsigned char instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.unsigned_char_var = self.UNSIGNED_CHAR_UPPER + 1

    def test_unsigned_short_get_upper_overflow(self):
        """ unsigned short virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.unsigned_short_get()
            self.uninstall_hook()

    def test_unsigned_short_set_upper_overflow(self):
        """ unsigned short function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.unsigned_short_set(
                    self.UNSIGNED_SHORT_UPPER + 1)

    def test_unsigned_short_var_upper_overflow(self):
        """ unsigned short instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.unsigned_short_var = self.UNSIGNED_SHORT_UPPER + 1

    def test_unsigned_int_get_upper_overflow(self):
        """ unsigned int virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.unsigned_int_get()
            self.uninstall_hook()

    def test_unsigned_int_set_upper_overflow(self):
        """ unsigned int function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.unsigned_int_set(
                    self.UNSIGNED_INT_UPPER + 1)

    def test_unsigned_int_var_upper_overflow(self):
        """ unsigned int instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.unsigned_int_var = self.UNSIGNED_INT_UPPER + 1

    def test_unsigned_long_get_upper_overflow(self):
        """ unsigned long virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.unsigned_long_get()
            self.uninstall_hook()

    def test_unsigned_long_set_upper_overflow(self):
        """ unsigned long function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.unsigned_long_set(
                    self.UNSIGNED_LONG_UPPER + 1)

    def test_unsigned_long_var_upper_overflow(self):
        """ unsigned long instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.unsigned_long_var = self.UNSIGNED_LONG_UPPER + 1

    def test_unsigned_long_long_get_upper_overflow(self):
        """ unsigned long long virtual result upper bound. """

        with self.assertRaises(OverflowError):
            self.install_hook()
            self.overflow_upper_fixture.unsigned_long_long_get()
            self.uninstall_hook()

    def test_unsigned_long_long_set_upper_overflow(self):
        """ unsigned long long function argument upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.unsigned_long_long_set(
                    self.UNSIGNED_LONG_LONG_UPPER + 1)

    def test_unsigned_long_long_var_upper_overflow(self):
        """ unsigned long long instance variable upper bound. """

        with self.assertRaises(OverflowError):
            self.overflow_upper_fixture.unsigned_long_long_var = self.UNSIGNED_LONG_LONG_UPPER + 1

    ###########################################################################
    # The following test bool convertors for valid values.
    ###########################################################################

    def test_bool_get_true(self):
        """ bool virtual result with a True value. """

        self.assertIs(self.true_fixture.bool_get(), True)

    def test_bool_set_true(self):
        """ bool function argument with a True value. """

        self.true_fixture.bool_set(True)

    def test_bool_var_true(self):
        """ bool instance variable with a True value. """

        self.true_fixture.bool_var = True

    def test_bool_get_false(self):
        """ bool virtual result with a True value. """

        self.assertIs(self.false_fixture.bool_get(), False)

    def test_bool_set_false(self):
        """ bool function argument with a False value. """

        self.false_fixture.bool_set(False)

    def test_bool_var_false(self):
        """ bool instance variable with a False value. """

        self.false_fixture.bool_var = False

    def test_bool_get_nonzero(self):
        """ bool virtual result with a non-zero value. """

        self.assertIs(self.nonzero_fixture.bool_get(), True)

    def test_bool_set_nonzero(self):
        """ bool function argument with a non-zero value. """

        self.nonzero_fixture.bool_set(-1)

    def test_bool_var_nonzero(self):
        """ bool instance variable with a non-zero value. """

        self.nonzero_fixture.bool_var = -1

    def test_bool_get_zero(self):
        """ bool virtual result with a zero value. """

        self.assertIs(self.zero_fixture.bool_get(), False)

    def test_bool_set_zero(self):
        """ bool function argument with a zero value. """

        self.zero_fixture.bool_set(0)

    def test_bool_var_zero(self):
        """ bool instance variable with a zero value. """

        self.zero_fixture.bool_var = 0



# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>


# The minimum required version of setuptools.  This is the earliest version
# that supports PEP 625.  Remember to update pyproject.toml in the root
# directory.
MINIMUM_SETUPTOOLS = '69.5'

# The oldest supported minor version of Python v3.  Remember to update
# pyproject.toml in the root directory.
OLDEST_SUPPORTED_MINOR = 9



# SPDX-License-Identifier: BSD-2-Clause

# Copyright (c) 2024 Phil Thompson <phil@riverbankcomputing.com>


from enum import Enum, Flag, IntEnum, IntFlag

from utils import SIPTestCase


class EnumsTestCase(SIPTestCase):
    """ Test the support for enums in ABI v13 and later. """

    @classmethod
    def setUpClass(cls):
        """ Set up the test case. """

        super().setUpClass()

        from .enums import EnumClass

        class NamedEnumFixture(EnumClass):
            """ A fixture for testing named enum values. """

            def __init__(self, value):
                """ Initialise the object. """

                super().__init__()

                self._value = value

            def named_virt(self):
                return self._value

        cls.member_fixture = NamedEnumFixture(
                EnumClass.ClassNamedEnum.ClassNamedMember)
        cls.int_fixture = NamedEnumFixture(0)

        class ScopedEnumFixture(EnumClass):
            """ A fixture for testing scoped enum values. """

            def scoped_virt(self):
                return EnumClass.ClassScopedEnum.ClassScopedMember

        cls.scoped_enum_fixture = ScopedEnumFixture()

    @classmethod
    def tearDownClass(cls):
        """ Tear down the test case. """

        # Remove all references to the extension module so that the superclass
        # can unload it.
        del cls.member_fixture
        del cls.int_fixture

        del cls.scoped_enum_fixture

        super().tearDownClass()

    ###########################################################################
    # The following test anonymous enums.
    ###########################################################################

    def test_ModuleAnon(self):
        """ Test a module level anonymous enum. """

        from .enums import AnonMember

        self.assertIsInstance(AnonMember, int)
        self.assertEqual(AnonMember, 10)

    def test_ClassAnon(self):
        """ Test a class level anonymous enum. """

        from .enums import EnumClass

        self.assertIsInstance(EnumClass.ClassAnonMember, int)
        self.assertEqual(EnumClass.ClassAnonMember, 40)

    ###########################################################################
    # The following test the /BaseType/ annotation.
    ###########################################################################

    def test_Enum_BaseType(self):
        """ Test /BaseType=Enum/. """

        from .enums import EnumBase

        self.assertTrue(issubclass(EnumBase, Enum))
        self.assertFalse(issubclass(EnumBase, Flag))
        self.assertFalse(issubclass(EnumBase, IntEnum))
        self.assertFalse(issubclass(EnumBase, IntFlag))

    def test_Flag_BaseType(self):
        """ Test /BaseType=Flag/. """

        from .enums import FlagBase

        self.assertTrue(issubclass(FlagBase, Flag))
        self.assertFalse(issubclass(FlagBase, IntEnum))
        self.assertFalse(issubclass(FlagBase, IntFlag))

    def test_IntEnum_BaseType(self):
        """ Test /BaseType=IntEnum/. """

        from .enums import IntEnumBase

        self.assertFalse(issubclass(IntEnumBase, Flag))
        self.assertTrue(issubclass(IntEnumBase, IntEnum))
        self.assertFalse(issubclass(IntEnumBase, IntFlag))

    def test_IntFlag_BaseType(self):
        """ Test /BaseType=IntFlag/. """

        from .enums import IntFlagBase

        self.assertFalse(issubclass(IntFlagBase, IntEnum))
        self.assertTrue(issubclass(IntFlagBase, IntFlag))

    ###########################################################################
    # The following test named enums.
    ###########################################################################

    def test_ModuleNamed(self):
        """ Test a module level named enum. """

        from .enums import NamedEnum

        self.assertTrue(issubclass(NamedEnum, Enum))
        self.assertEqual(NamedEnum.NamedMember.value, 20)

    def test_ClassNamed(self):
        """ Test a class level named enum. """

        from .enums import EnumClass

        self.assertTrue(issubclass(EnumClass.ClassNamedEnum, Enum))
        self.assertEqual(EnumClass.ClassNamedEnum.ClassNamedMember.value, 50)

    def test_named_get_member(self):
        """ named enum virtual result with a member value. """

        from .enums import EnumClass

        self.install_hook()
        self.assertEqual(self.member_fixture.named_get(),
                EnumClass.ClassNamedEnum.ClassNamedMember)
        self.uninstall_hook()

    def test_named_set_member(self):
        """ named enum function argument with a member value. """

        from .enums import EnumClass

        self.member_fixture.named_set(
                EnumClass.ClassNamedEnum.ClassNamedMember)

    def test_named_var_member(self):
        """ named enum instance variable with a member value. """

        from .enums import EnumClass

        self.member_fixture.named_var = EnumClass.ClassNamedEnum.ClassNamedMember

    def test_named_overload_set(self):
        """ overloaded named enum function argument. """

        from .enums import EnumClass

        self.member_fixture.named_overload_set(
                EnumClass.ClassNamedEnum.ClassNamedMember)
        self.assertTrue(self.member_fixture.named_overload)

    def test_named_get_int(self):
        """ named enum virtual result with an integer value. """

        from .enums import EnumClass

        with self.assertRaises(TypeError):
            self.install_hook()
            self.int_fixture.named_get()
            self.uninstall_hook()

    def test_named_set_int(self):
        """ named enum function argument with an integer value. """

        with self.assertRaises(TypeError):
            self.int_fixture.named_set(50)

    def test_named_var_int(self):
        """ named enum instance variable with an integer value. """

        with self.assertRaises(TypeError):
            self.int_fixture.named_var = 50

    ###########################################################################
    # The following test scoped enums.
    ###########################################################################

    def test_ModuleScoped(self):
        """ Test a module level C++11 scoped enum. """

        from .enums import ScopedEnum

        self.assertTrue(issubclass(ScopedEnum, Enum))
        self.assertEqual(ScopedEnum.ScopedMember.value, 30)

    def test_ClassScoped(self):
        """ Test a class level C++11 scoped enum. """

        from .enums import EnumClass

        self.assertTrue(issubclass(EnumClass.ClassScopedEnum, Enum))
        self.assertEqual(EnumClass.ClassScopedEnum.ClassScopedMember.value, 70)

    def test_scoped_get_member(self):
        """ scoped enum virtual result with a member value. """

        from .enums import EnumClass

        self.install_hook()
        self.assertIs(self.scoped_enum_fixture.scoped_get(),
                EnumClass.ClassScopedEnum.ClassScopedMember)
        self.uninstall_hook()

    def test_scoped_set_member(self):
        """ scoped enum function argument with a member value. """

        from .enums import EnumClass

        self.scoped_enum_fixture.scoped_set(
                EnumClass.ClassScopedEnum.ClassScopedMember)

    def test_scoped_var_member(self):
        """ scoped enum instance variable with a member value. """

        from .enums import EnumClass

        self.scoped_enum_fixture.scoped_var = EnumClass.ClassScopedEnum.ClassScopedMember
